var AbstractionLayer = MindFusion.AbstractionLayer;
var AnchorPattern = MindFusion.Diagramming.AnchorPattern;
var AnchorPoint = MindFusion.Diagramming.AnchorPoint;
var DiagramNode = MindFusion.Diagramming.DiagramNode;
var DiagramLink = MindFusion.Diagramming.DiagramLink;
var ContainerNode = MindFusion.Diagramming.ContainerNode;
var ShapeNode = MindFusion.Diagramming.ShapeNode;
var Style = MindFusion.Diagramming.Style;
var Theme = MindFusion.Diagramming.Theme;
var FontStyle = MindFusion.Drawing.FontStyle;
var Font = MindFusion.Drawing.Font;
var Alignment = MindFusion.Diagramming.Alignment;
var HandlesStyle = MindFusion.Diagramming.HandlesStyle;
var Events = MindFusion.Diagramming.Events;
var Diagram = MindFusion.Diagramming.Diagram;
var NodeListView = MindFusion.Diagramming.NodeListView;
var Rect = MindFusion.Drawing.Rect;
var Shape = MindFusion.Diagramming.Shape;
var DashStyle = MindFusion.Drawing.DashStyle;
var Point = MindFusion.Drawing.Point;
var FlowchartLayout = MindFusion.Graphs.FlowchartLayout;
var Size = MindFusion.Drawing.Size;

var diagram, nodeList;
var backgroundColor, linkDashStyle, baseShape, headShape, headBrush;
var colors;
var steps;
var labelNodes;

window.onload = function(e){ 

	backgroundColor = "#806fb7";
    //initialize the color picker
	var colorBkgr = document.querySelector("#colorBkgr");
	colorBkgr.value = backgroundColor;
	colorBkgr.addEventListener("input", updateBackground, false);
	colorBkgr.addEventListener("change", updateBackground, false);

    //initialize all necessary colors
   	colors = ['#fdb44f', '#de7e00', '#8b2413', '#dc391e', '#806fb7', '#514181', '#254b73', '#3c79b8', '#17a050', '#0b5028'];
	steps = ['Plan a route', 'Choose a guide', 'Make an announcement', 'Gather the group', 'Choose start date', 'Prepare equipment', 'Start hiking', 'Log progress', 
	'Reach destination', 'Celebrate the achievment'];
	labelNodes = [];
    
	var diagramEl = document.getElementById('diagram');
	// create a Diagram component that wraps the "diagram" canvas
	diagram = AbstractionLayer.createControl(Diagram, null, null, null, diagramEl);
	diagram.setAllowInplaceEdit(true);
	diagram.setRouteLinks(true);
	diagram.setShowGrid(true);
	diagram.setRoundedLinks(true);
	diagram.setDefaultShape('Ellipse');
	diagram.setShadowsStyle(MindFusion.Diagramming.ShadowsStyle.None);
	diagram.setBounds(new Rect(0, 0, 2000,2000));
	
	//create a theme for initial styling of diagram nodes and links
	var theme = new Theme();	
	
	 var shapeNodeStyle = new Style();
    shapeNodeStyle.setBrush({ type: 'SolidBrush', color: insertAlpha(backgroundColor, 0.5) });
    shapeNodeStyle.setStroke(backgroundColor);
	shapeNodeStyle.setStrokeThickness(10);
    shapeNodeStyle.setTextColor("#202020");
    shapeNodeStyle.setFontName("Times New Roman");
	shapeNodeStyle.setFontStyle(MindFusion.Drawing.FontStyle.Bold);
    shapeNodeStyle.setFontSize(4);
    shapeNodeStyle.setBackBrush("#e0e9e9"); 
    theme.styles["std:ShapeNode"] = shapeNodeStyle; 
	
	var linkStyle = new Style();
	linkStyle.setStroke("#a6a6a6");
	linkStyle.setTextColor("#585A5C");
	linkStyle.setFontName("Times New Roman");
	linkStyle.setFontSize(4);
	theme.styles["std:DiagramLink"] = linkStyle;
	diagram.setTheme(theme);	
	
	//wire the node created and link created events
	diagram.addEventListener(Events.linkCreated, onLinkCreated);
	diagram.addEventListener(Events.nodeCreated, onNodeCreated);
	
	var nodeListEl = document.getElementById('nodeList');
	// create an NodeListView component that wraps the "nodeList" canvas
	nodeList = AbstractionLayer.createControl(NodeListView, null, null, null, nodeListEl);	
	nodeList.setIconSize(new Size(30,30));
	nodeList.setDefaultNodeSize(new Size(10,10));
	nodeList.addEventListener(Events.nodeSelected, onShapeSelected);
	//transparent text node
	 var node = new ShapeNode();	
	 node.setTransparent(true);
	 node.setShape("Rectangle");
	 node.setText("Text");
	 node.setTextColor("#7F7F7F");
	 node.setFont(new Font("Verdana", 12, true, false));
	 nodeList.addNode(node, "Text");  
    //ellipse node with thick border
	node = new MindFusion.Diagramming.ShapeNode(diagram);
	node.setShape("Ellipse");
	node.setStroke(backgroundColor);
	node.setStrokeThickness(10.0);
	node.setBrush({ type: 'SolidBrush', color: insertAlpha(backgroundColor, 0.5)});
	nodeList.addNode(node,"Ellipse");	 
	 //arrow node for connecting the nodes
    node = new MindFusion.Diagramming.ShapeNode(diagram);
	node.setShape("Arrow3");
	node.setBrush({ type: 'SolidBrush', color: backgroundColor });
	node.setStrokeThickness(1);
	node.setStroke(backgroundColor);
	nodeList.addNode(node, "Arrow");	 	
	
	onLoaded();

};

//inserts the given alpha value to the provided color
function insertAlpha(color, alpha)
{	//the color is in hex format - convert it to rgb
	var rgbValue = hexToRgb(color);
	
	if(rgbValue)
	{
		var result = "rgba(" + rgbValue.r + "," + rgbValue.g + "," + rgbValue.b + "," + alpha + ")";	
		return result;
	}		
	
}

//return the RGB value of a color in hexadecimal format
function hexToRgb(hex) {
  var result = /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.exec(hex);
  return result ? {
    r: parseInt(result[1], 16),
    g: parseInt(result[2], 16),
    b: parseInt(result[3], 16)
  } : null;
}

//create the diagram that you see when the page loads
function onLoaded()
{
	var counter = 0;
	
	//the nodes follow the pattern of a cosine curve
	for(i=120; i<=420; i+=32){   
		
		x = i*0.9 - 80;
		y =  65 - Math.cos(i*(Math.PI/180)) * 50;	
     
		//console.log(x,y);
		
		var node = diagram.getFactory().createShapeNode(new Rect(x, y, 17, 17));
		node.setStroke(colors[counter]);
		node.setStrokeThickness(10.0);
		node.setBrush({ type: 'SolidBrush', color: insertAlpha(colors[counter], 0.5)});
		
		//create a transparent text nodes for each node
		var textNode = diagram.getFactory().createShapeNode(new Rect(x - 5, y + 40, 30, 10));
		textNode.setText(steps[counter]);
		textNode.setTextColor('#7F7F7F');
		textNode.setTransparent(true);
		
		//conect the node and the text represented by another node
		var link = diagram.getFactory().createDiagramLink(textNode, node);	
		link.setHeadShape(null);		
	    link.setHeadBrush({ type: 'SolidBrush', color: '#7F7F7F' });		
		
		counter++;		
		node.setText("Step " + counter);
		node.setTextColor('#202020');
		node.setFont(new MindFusion.Drawing.Font("Times New Roman", 4, true, false));
    }
	
	var nodes = diagram.getNodes();
	
	//connect all nodes with thick links
	for(var i = 0; i < nodes.length - 2; i+=2)
	{
		var link = diagram.getFactory().createDiagramLink(nodes[i], nodes[i+2]);	
		link.setStroke(link.getOrigin().getStroke());	
	    link.setHeadBrush({ type: 'SolidBrush', color: link.getOrigin().getStroke()});		
		link.setStrokeThickness(8);
		
	}

}

//remove all items from the diagram
function clearItems()
{
	diagram.clearAll();
}

//save the diagram to the local storage
function save()
{
	localStorage.setItem('mf_dgr', diagram.toJson());
}

//loads the diagram from the local storage
function load()
{
	var dgr = localStorage.getItem('mf_dgr');
	if(dgr)
		diagram.fromJson(dgr);
}

//update the default color for diagram shapes. Change the color to the selected shape.
function updateBackground(event) {
   backgroundColor = event.target.value;
   var selectedItem = diagram.selection.items[0];
		if(selectedItem)
		{	//if the selected shape is an ellipse - change its color and the color of its outgoing links
			if(selectedItem.getShape().getId() === 'Ellipse')
			{
				selectedItem.setBrush({ type: 'SolidBrush', color: insertAlpha(backgroundColor, 0.5) });
				selectedItem.setStroke(backgroundColor);
				
				for(var i = 0; i < selectedItem.getOutgoingLinks().length; i++)
				{
					var link = selectedItem.getOutgoingLinks()[i];
					
					link.setStroke(backgroundColor);	
					link.setHeadBrush({ type: 'SolidBrush', color: backgroundColor });		
				}
			}
			else
			{    //the other shapes ust get a new solid color
				selectedItem.setBrush({ type: 'SolidBrush', color: backgroundColor });
				selectedItem.setStroke(backgroundColor);	
			}
			
		}
}


//handles the nodeCreated event
function onNodeCreated(sender, args)
{
	var node = args.getNode();
	node.setStroke(backgroundColor);
	
	//if the created node is an ellipse, its brush must be semi-transparent and the stroke - thick.
	if(node.getShape().getId() === 'Ellipse')
	{	
		node.setBrush({ type: 'SolidBrush', color: insertAlpha(backgroundColor, 0.5)});		
		node.setStrokeThickness(10.0);
	}
	else
	{ //if it is a text node - the color of text should be light gray
	  node.setBrush({ type: 'SolidBrush', color: backgroundColor }); 
	  node.setTextColor('#7F7F7F');
	}	
}

//handle the linkCreated event
function onLinkCreated(sender, args)
{
	var link = args.getLink();
	//if the link comes from an ellipse, its color must be the color of the node
	if(link.getOrigin().getShape().getId() === 'Ellipse')
	{
		link.setStroke(link.getOrigin().getStroke());	
	    link.setHeadBrush({ type: 'SolidBrush', color: link.getOrigin().getStroke()});		
		link.setStrokeThickness(8);//if it comes from a text node, it must not have arrow heads
	}else if(link.getOrigin().getShape().getId() === 'Rectangle')
	{
		link.setHeadShape(null);	
		link.setBaseShape(null);		
	    link.setHeadBrush({ type: 'SolidBrush', color: '#7F7F7F' });		
	}	
	
	link.setTextAlignment(MindFusion.Diagramming.Alignment.Near);
}

//when a shape is selected, it is set as the default for the diagram
function onShapeSelected(sender, e)
{
	var selectedNode = e.getNode();
	if (selectedNode)
		diagram.setDefaultShape(selectedNode.getShape());
}



























































